const debug = require('debug')('ets:provider');
const EtherscanProvider = require('./provider');

class EtherscanHTTP extends EtherscanProvider {
  /**
   * @param {string} address
   */
  async start(
    address,
    startblock = EtherscanHTTP.DEFAULT_START_BLOCK,
    endblock = EtherscanHTTP.DEFAULT_END_BLOCK,
    ERC20TokenAddress = null
  ) {
    this.baseURL = this.config.baseURL;
    const { url, method, params, data } = this.config
      .listTx(address, startblock, endblock);
    
    const txs = await this.request(url, method, params, data);

    if (ERC20TokenAddress && txs.length > 0) {
      const blocks = txs.map(tx => parseInt(tx.blockNumber));
      const logs = await this.getERC20MintLogs(
        ERC20TokenAddress,
        Math.min(...blocks),
        Math.max(...blocks)
      );

      for (let tx of txs) {
        const { hash } = tx;

        tx[EtherscanHTTP.LOGS_KEY] = [];

        for (let log of logs) {
          const { transactionHash } = log;

          if (hash === transactionHash) {
            tx[EtherscanHTTP.LOGS_KEY].push(log);
          }
        }

        debug(
          'logs',
          tx[EtherscanHTTP.LOGS_KEY]
            ? tx[EtherscanHTTP.LOGS_KEY].length
            : 0
        );
      }
    }
    
    return this.tryPushAndClose(...txs);
  }

  /**
   * Get logs for ERC20 token mint transfers
   * @param {string} address 
   * @param {number} startblock 
   * @param {number} endblock 
   */
  async getERC20MintLogs(address, fromBlock, toBlock) {
    const { url, method, params, data } = this.config
        .listERC20Transfers(address, fromBlock, toBlock, true);

    return this.request(url, method, params, data);
  }

  /**
   * Transform response in items
   * @param {*} response 
   */
  async transform(response) {
    return response.data.result || [];
  }

  /**
   * Logs key
   */
  static get LOGS_KEY() {
    return '$logs';
  }

  /**
   * Default tx start block
   */
  static get DEFAULT_START_BLOCK() {
    return 0;
  }

  /**
   * Default tx end block
   */
  static get DEFAULT_END_BLOCK() {
    return 99999999999999;
  }
}


module.exports = EtherscanHTTP;
